# -*- coding: utf-8 -*-
"""Prediccion de asistentes y aprobados en CADP.ipynb

Automatically generated by Colaboratory.

Original file is located at
    https://colab.research.google.com/drive/15i8dPidUKUuqnLjQVrf-HMHnTsUfKLVE

# Modelos predictivos para CADP

* Licencia: CC-BY-NC-SA https://creativecommons.org/licenses/by-nc-sa/4.0/deed.es
* Autor: Dr. Gonzalo . Villarreal, PREBI-SEDICI (UNLP) y CESGI (CIC) https://orcid.org/0000-0002-3602-8211

## Descipción

Esta notebook contiene 2 modelos para predecir asistencia y tasa de aprobación en los parciales de CADP . Uno de los modelos se basa en regresión linea, y otro de los modelos se basa en regresión polinomial (gr=3) .

Para ejecutar las predicciones, deben ejecutarse las 3 celdas de código:
- La primera inicializa todas las variables (observar la variable "nueva_caracteristicas_presentes" , que es la que se utilizará para realizar las predicciones)
- La segunda ejecuta los modelos de regresión lineal
- La tercera ejecuta los modelos de regresión polynomial

# Inicialización

Cargamos el dataset, e inicializamos algunas variables que se utilizaran en todos los modelos
"""

import pandas as pd
import numpy as np

from sklearn.linear_model import LinearRegression

dir = '/dataset/'
#dir = '/content/drive/MyDrive/Colab Notebooks/'
#file= 'cadp_1st_exam.csv'
file='cadp.csv'
data = pd.read_csv(dir + file,decimal=",")

# caracteristicas que se utilizaran para predecir presentismo
target_presentes = '% presentes'

# datos que se utilizaran para realizar la predicción. Vamos a predecir los presentes para: cuatrimestre 1 y parcial 3
features_presentes = ['Cuatrimestre','Parcial']
nuevas_caracteristicas_presentes = np.array([1, 3])

print("Parámetros de los modelos predictivos:")
print("Cuatrimestre: " , nuevas_caracteristicas_presentes[0], " - Exman ", nuevas_caracteristicas_presentes[1])
print("---------------------------------")

# caracteristicas que se utilizaran para predecir tasa de aprobacion
features_aprobados = ['Cuatrimestre','Parcial','% presentes']
target_aprobados = '% aprobados'

# OPCION 2: utilizamos todas las columnas del dataset para realizar predicciones, aunque no tiene mucho sentido porque hay datos que no podemos estimar
#features_presentes = ['Cuatrimestre','Parcial','% aprobados',	'% desaprobados',	'% insuficientes']
#nuevas_caracteristicas_presentes = np.array([1, 2, 0.21,	0.37,	0.42])
# caracteristicas que se utilizaran para predecir tasa de aprobacion
#features_aprobados = ['Cuatrimestre','Parcial','% presentes','% aprobados',	'% desaprobados',	'% insuficientes']
#target_aprobados = '% aprobados'

"""## Modelos de regresión lineal

Veamos si podemos predecir el porcentaje de presentes y el porcentaje de aprobados sobre esos presentes



"""
print("-------------------------------")
print("Modelo basado en Regresión Lineal")
# Dividir los datos en características (X) y objetivo (y)
X = data[features_presentes]
y = data[target_presentes]

# Crear y ajustar el modelo de regresión lineal
model = LinearRegression()
model.fit(X, y)
prediccion = model.predict([nuevas_caracteristicas_presentes])
print("Predicción:", round(prediccion[0]*100,2), "% de asistencia")

# Ahora vamos a prededir los aprobados usando RL

#Dividir los datos en características (X) y objetivo (y)
X = data[features_aprobados]
y = data[target_aprobados]
model_aprobacion = LinearRegression()
model_aprobacion.fit(X, y)

nuevas_caracteristicas_aprobados = np.append(nuevas_caracteristicas_presentes, prediccion)

# Ejemplo de valores de características para hacer una predicción: cuatrimestre 1, parcial 3, presentes predichos previamente
prediccion_aprobados = model_aprobacion.predict([nuevas_caracteristicas_aprobados])
print("Predicción:", round(prediccion_aprobados[0]*100,2), "% de aprobados")

"""## Modelos polinomiales

Vamos a construir un polinomio de grado 3 para ver si sus predicciones mejoran las predicciones del modelo lineal
"""

from sklearn.linear_model import LinearRegression
from sklearn.preprocessing import PolynomialFeatures

print("-------------------------------")
print("Modelo basado en Polinomio de grado 3")

# Dividir los datos en características (X) y objetivo (y)
X = data[features_presentes]
y = data[target_presentes]

# Ajustamos un modelo polinomial de grado 3 para más INRI
poly_features = PolynomialFeatures(degree=3)
X_poly = poly_features.fit_transform(X)

# Generar una nueva observación para predecir
nueva_observacion = [nuevas_caracteristicas_presentes]  # Valores de las características anteriores

# Transformar la nueva observación en características polinomiales
nueva_observacion_poly = poly_features.transform(nueva_observacion)
lin2 = LinearRegression();
lin2.fit(X_poly,y)

# Realizar la predicción
prediccion_presentes = lin2.predict(nueva_observacion_poly)
print("Predicción:", round(prediccion_presentes[0]*100,2), "% de asistencia")

# Nuevamente, utilizamos la predicción anterior para ver si podemos predecir los aprobados

X = data[features_aprobados]
y = data[target_aprobados]

# Ajustamos un modelo polinomial de grado 3 para más INRI
poly_features = PolynomialFeatures(degree=3)
X_poly = poly_features.fit_transform(X)

# Generar una nueva observación para predecir
nueva_observacion = np.append(nuevas_caracteristicas_presentes, prediccion_presentes)
nueva_observacion_poly = poly_features.transform([nueva_observacion])
lin2 = LinearRegression();
lin2.fit(X_poly,y)

# Realizar la predicción
prediccion_aprobados = lin2.predict(nueva_observacion_poly)
print("Predicción:", round(prediccion_aprobados[0]*100,2), "% de aprobados")

